'１次元熱伝導解析V50の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/04)
Imports System.Drawing.Printing

Module Th1DCom
    '1次元の非定常熱伝導解析
    '両端は外部境界・・・断熱、温度固定、熱伝達、熱流束、熱放射の5種類
    '複数層（各均等ﾒｯｼｭ）
    '各層材料とも固体・液体区分物性（含潜熱）、ただし密度は固定

    'V41での改良点・・・後処理の解析結果の主要値表示でクリップボードにコピーを追加
    'V42での改良点・・・各画面でボタンを読みやすい色に改善
    'V43での改良点・・・Visual Basic 2017に変更
    'V44での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V45での改良点・・・ファイル入出力画面でキャンセル処理の改善
    'V46での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V47での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V48での改良点・・・非定常のグラフデータの作成を追加、
    '                   材料データの入力で、密度だけを変更できないバグを修正
    'V49での改良点・・・計算時間間隔の上限の有無選択を廃止、
    '                   計算時間間隔の自動改善を組込んで所要時間を大幅短縮
    'V50での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fgraph$                  'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public Fm$                      '入力データのファイル名
    Public Fy$                      '計算結果のファイル名
    Public Title$                   'タイトル

    Public Timst As Date            '計算開始時刻

    Public Idcycle As Integer       '出力サイクル間隔
    Public InitData As Integer      '初期値データの入力区分（-1:なし、0:各層一括、
    '                                1:各点個別、2:結果ファイル読み込み）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public Kcycle As Integer        '作画指定のサイクル番号
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public Kindex As Integer        'ﾃｷｽﾄｱﾚｲのｲﾝﾃﾞｯｸｽ
    Public KindPost As Integer      '後処理の種類（1:等温線図、2:等温線図ｽﾗｲﾄﾞｼｮｰ、3:主要値表示、
    '                                4:非定常のｸﾞﾗﾌﾃﾞｰﾀ）
    Public KL1 As Integer           '作画の開始層番号
    Public KL2 As Integer           '作画の終了層番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumMax As Integer       '節点番号の最大値
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Kstep As Integer         '作画指定のステップ番号
    Public Ktime As Integer         '出力メッシュ番号
    Public MeshMax As Integer       '層の最大メッシュ数
    Public Mstat As Integer         '解析種別（0:定常、1:非定常）
    Public Nall As Integer          '変数の総数
    Public Ncycle As Integer        '境界条件のサイクル数
    Public NPgraph As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの節点数
    Public NL As Integer            '層の数
    Public Nline As Integer         'ﾌｧｲﾙの行数
    Public Nmat As Integer          '材料データ数
    Public NMdivMax As Integer      '材料の物性区分数最大値
    Public Nstep As Integer         '境界条件のステップ数
    Public Ntemp As Integer         '温度分割数
    Public Ntime As Integer         '非定常の出力時間メッシュ数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DtempMax As Single       '非定常の計算1回の温度変化許容値
    Public Dtime As Single          '非定常の出力の計算時間間隔
    Public DtimeInit As Single      '非定常の計算時間間隔の初期値
    Public TempMin As Single        '最低温度
    Public TempMax As Single        '最高温度
    Public TimeMax As Single        '非定常の打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public WidthL As Single         '伝熱体の幅（作画用）
    Public Xlower As Single         '作画領域のX最小値
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public Ylower As Single         '作画領域のY最小値
    Public Yupper As Single         '作画領域のY最大値

    Public ArrText As New ArrayList()

    Public UFileA(6) As String      '最近使用した入力ﾌｧｲﾙの保存配列

    Public KindBoundL() As Integer  '左端の境界種類（0:断熱、1:温度固定、2:熱伝達、3:熱流束、4:熱放射）
    Public KindBoundR() As Integer  '右端の境界種類（0:断熱、1:温度固定、2:熱伝達、3:熱流束、4:熱放射）
    Public KPgraph(20, 2) As Integer    'ｸﾞﾗﾌ点位置（層番号とメッシュ番号）
    Public KstartL() As Integer     '各層第1ﾒｯｼｭの対応変数番号
    Public Mesh() As Integer        '各層のメッシュ数
    Public NMdiv() As Integer       '材料の物性区分数
    Public NumMat() As Integer      '各層の材料種類番号

    Public BoundDataL(,) As Single  '左端境界の条件データ
    Public BoundDataR(,) As Single  '左端境界の条件データ
    Public Cspec(,) As Single       '材料の比熱
    Public Rlamda(,) As Single      '材料の熱伝導率
    Public Rlength() As Single      '各層の長さ
    Public Rou() As Single          '材料の密度
    Public TempInit(,) As Single    '初期温度
    Public TempLB() As Single       '層左の層境界温度
    Public TempMat(,) As Single     '材料の区分上限温度
    Public TempSurf(2) As Single    '両端境界表面温度
    Public TimeStep() As Single     'ステップ時間
    Public Tu() As Single           '差分計算の解
    Public Xdraw(4), Ydraw(4) As Single     '色塗り多角形の座標
    Public Xmouse(), Ymouse() As Single     'ﾏｳｽ位置座標

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        'ii･･･ﾌｧｲﾙ番号
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Therm1D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NL = dat(1)
        ReDim Mesh(NL), NumMat(NL), Rlength(NL)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        ReDim NMdiv(Nmat), Rou(Nmat)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nstep = dat(1)
        ReDim TimeStep(Nstep)
        ReDim KindBoundL(Nstep), BoundDataL(Nstep, 2)
        ReDim KindBoundR(Nstep), BoundDataR(Nstep, 2)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        WidthL = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        InitData = dat(1)
        aa$ = sr.ReadLine
        For i = 1 To NL
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            Rlength(i) = dat(2)
            Mesh(i) = dat(3)
        Next i
        aa$ = sr.ReadLine
        NMdivMax = 0
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NMdiv(i) = dat(1)
            If NMdiv(i) > NMdivMax Then
                NMdivMax = NMdiv(i)
            End If
        Next i
        ReDim Cspec(Nmat, NMdivMax), Rlamda(Nmat, NMdivMax)
        ReDim TempMat(Nmat, NMdivMax)
        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Rou(i) = dat(1)
            aa$ = sr.ReadLine
            For j = 1 To NMdiv(i)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Cspec(i, j) = dat(1)
                Rlamda(i, j) = dat(2)
                TempMat(i, j) = dat(3)
            Next j
        Next i

        If Nstep > 1 Then
            aa$ = sr.ReadLine
            For i = 1 To Nstep
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TimeStep(i) = dat(1)
            Next i
        End If

        aa$ = sr.ReadLine
        For i = 1 To 2
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For j = 1 To Nstep
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                If i = 1 Then
                    KindBoundL(j) = dat(1)
                    BoundDataL(j, 1) = dat(2)
                    BoundDataL(j, 2) = dat(3)
                Else
                    KindBoundR(j) = dat(1)
                    BoundDataR(j, 1) = dat(2)
                    BoundDataR(j, 2) = dat(3)
                End If
            Next j
        Next i

        For i = 1 To NL
            If i = 1 Then
                MeshMax = Mesh(i)
            ElseIf Mesh(i) > MeshMax Then
                MeshMax = Mesh(i)
            End If
        Next i

        If InitData > 0 Then
            ReDim TempInit(NL, MeshMax)
            aa$ = sr.ReadLine
            For i = 1 To NL
                aa$ = sr.ReadLine
                For j = 1 To Mesh(i)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    TempInit(i, j) = dat(1)
                Next j
                TempInit(i, 0) = TempInit(i, 1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)

        aa$ = "プログラム名" + "," + "Therm1D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "層の数" + "," + Str(NL)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "境界条件のステップ数" + "," + Str(Nstep)
        sw.WriteLine(aa$)
        aa$ = "伝熱体の幅（作画用）" + "," + Str(WidthL)
        sw.WriteLine(aa$)
        aa$ = "初期値データの有無" + "," + Str(InitData)
        sw.WriteLine(aa$)

        aa$ = "各層の形状データ（層番号、材料番号、長さ、分割メッシュ数）"
        sw.WriteLine(aa$)
        For i = 1 To NL
            aa$ = "  " + Str$(i) + ",  " + Str$(NumMat(i))
            aa$ = aa$ + ",  " + Str$(Rlength(i)) + ",  " + Str$(Mesh(i))
            sw.WriteLine(aa$)
        Next i
        sw.WriteLine("各材料の物性区分数（材料番号、区分数)")
        For i = 1 To Nmat
            aa$ = "  " + Str$(i) + ",  " + Str$(NMdiv(i))
            sw.WriteLine(aa$)
        Next i
        sw.WriteLine("各材料の物性データ")
        For i = 1 To Nmat
            aa$ = "材料番号" + "," + Str(i)
            sw.WriteLine(aa$)
            aa$ = "  密度" + "," + Str(Rou(i))
            sw.WriteLine(aa$)
            sw.WriteLine("  区分番号、比熱、熱伝導率、上限温度")
            For j = 1 To NMdiv(i)
                aa$ = "    " + Str$(j) + ",  " + Str$(Cspec(i, j))
                aa$ = aa$ + ",  " + Str$(Rlamda(i, j))
                aa$ = aa$ + ",  " + Str$(TempMat(i, j))
                sw.WriteLine(aa$)
            Next j
        Next i

        If Nstep > 1 Then
            sw.WriteLine("境界条件のステップ時間（ステップ番号、時間）")
            For i = 1 To Nstep
                aa$ = Str$(i) + ",  " + Str$(TimeStep(i))
                sw.WriteLine(aa$)
            Next i
        End If

        sw.WriteLine("両端境界のデータ")
        For i = 1 To 2
            aa$ = "境界番号" + "," + Str(i)
            sw.WriteLine(aa$)
            aa$ = "　ステップ番号、境界種類（0:断熱、1:温度固定、2:熱伝達、3:熱流束、4:熱放射）、各データ"
            sw.WriteLine(aa$)
            For j = 1 To Nstep
                aa$ = Str(j)
                If i = 1 Then
                    aa$ = aa$ + ",  " + Str(KindBoundL(j))
                    aa$ = aa$ + ",  " + Str(BoundDataL(j, 1))
                    aa$ = aa$ + ",  " + Str(BoundDataL(j, 2))
                Else
                    aa$ = aa$ + ",  " + Str(KindBoundR(j))
                    aa$ = aa$ + ",  " + Str(BoundDataR(j, 1))
                    aa$ = aa$ + ",  " + Str(BoundDataR(j, 2))
                End If
                sw.WriteLine(aa$)
            Next j
        Next i

        If InitData > 0 Then
            sw.WriteLine("初期温度データ（メッシュ番号、温度）")
            For i = 1 To NL
                aa$ = "層番号" + "," + Str(i)
                sw.WriteLine(aa$)
                For j = 1 To Mesh(i)
                    aa$ = Str$(j) + ",  " + Str$(TempInit(i, j))
                    sw.WriteLine(aa$)
                Next j
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Th1DMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Th1DMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.Close()

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i)
            xydraw(i - 1).Y = Ydraw(i)
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PDoc_PrintPage(ByVal sender As System.Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs)
        Dim linesPerPage As Integer
        Dim ypos As Single
        Dim ic, ncount As Integer
        Dim xleftMargin As Single
        Dim ytopMargin As Single
        Dim lineText$
        Dim printfont As Font = New Font("ＭＳ ゴシック", 10)

        linesPerPage = e.MarginBounds.Height / printfont.GetHeight(e.Graphics)
        ypos = 0
        ncount = 0
        xleftMargin = e.MarginBounds.Left
        ytopMargin = e.MarginBounds.Top

        ic = 0
        While ncount < linesPerPage
            If Kindex < Nline Then
                lineText$ = ArrText(Kindex)
                Kindex = Kindex + 1
            Else
                ic = 1
                Exit While
            End If
            ypos = ytopMargin + ncount * printfont.GetHeight(e.Graphics)
            e.Graphics.DrawString(lineText$, printfont, Brushes.Black, xleftMargin, ypos, New StringFormat())
            ncount = ncount + 1
        End While

        If ic = 0 And Kindex < Nline Then
            e.HasMorePages = True
        Else
            e.HasMorePages = False
        End If

    End Sub

    Sub Rounding(ByVal kind As Integer, ByVal kfnum As Integer, _
        ByRef temp As Single)
        '丸め処理
        'kind･･･丸め種類（0:下丸め、1:上丸め）
        'kfnum･･･小数点以下の桁数
        'temp･･･温度
        Dim b1, b2, b3, b4 As Single
        b1 = 10 ^ kfnum
        b3 = temp * b1
        b4 = Int(b3 / b1)
        b2 = b3 - b1 * b4
        If b2 = 0 Then
            temp = CInt(b3) / b1
        ElseIf kind > 0 Then
            temp = Int(b3 + 1) / b1
        Else
            temp = Int(b3) / b1
        End If

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Therm1D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Therm1D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th1DUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th1DUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        If Cfix < 1 Then
            xb = xb * XYscale + Xlower
            yb = Yupper - yb * XYscale * Cfix
        Else
            xb = xb * XYscale / Cfix + Xlower
            yb = Yupper - yb * XYscale
        End If
    End Sub

End Module
